import { getUserInfo } from '@/api';
import router from '@/router';
import type { UserInfo } from '@/schemas/response';
import { type idAndNameSchema } from '@/schemas/response';
import type { MaybePromise } from '@/utils/types';
import { StorageSerializers, useLocalStorage } from '@vueuse/core';
import { ElMessage } from 'element-plus';
import { defineStore } from 'pinia';
import { computed, ref, watch } from 'vue';
import { z } from 'zod';

type Permission = z.infer<typeof idAndNameSchema>;
export const useUserStore = defineStore('user', () => {
  const token = useLocalStorage<string | null>('token', null, {
    serializer: StorageSerializers.string,
  });
  const userInfo = useLocalStorage<UserInfo | null>('user-info', null, {
    serializer: StorageSerializers.object,
  });
  const permissions = computed(
    () =>
      new Map<number, Permission>(
        (userInfo.value?.auth.permissions ?? []).map((permission) => [permission.id, permission]),
      ),
  );
  const clubPermissions = computed(
    () =>
      new Map<number, Permission>(
        (userInfo.value?.clubAuth?.permissions ?? []).map((permission) => [
          permission.id,
          permission,
        ]),
      ),
  );
  const initializing = ref(false);
  const logined = computed(() => (userInfo.value && userInfo.value.id !== -1) ?? false);
  watch(
    userInfo,
    (info, old) => {
      router.push({
        path:
          info && (info.id !== -1 || !old || old.id === -1)
            ? router.currentRoute.value.fullPath
            : '/',
        force: true,
      });
    },
    { flush: 'sync' },
  );
  const isSelf = (id: string | number) => userInfo.value?.id === Number(id);
  async function updateSelfUserInfo(): Promise<boolean> {
    initializing.value = true;
    try {
      const resp = await getUserInfo();
      if (!resp) return false;
      userInfo.value = resp.data;
      return true;
    } finally {
      initializing.value = false;
    }
  }

  const logoutInterceptors = new Set<LogoutInterceptor>();
  function addLogoutInterceptor(fn: LogoutInterceptor) {
    logoutInterceptors.add(fn);
  }
  function removeLogoutInterceptor(fn: LogoutInterceptor) {
    logoutInterceptors.delete(fn);
  }
  async function logout() {
    const res = await Promise.all(Array.from(logoutInterceptors).map((fn) => fn()));
    if (!res.every((r) => r === undefined || r)) return false;
    token.value = null;
    await updateSelfUserInfo();
    ElMessage.info('退出成功');
    return true;
  }
  return {
    token,
    userInfo,
    logined,
    permissions,
    clubPermissions,
    initializing,
    isSelf,
    updateSelfUserInfo,
    logoutInterceptors,
    addLogoutInterceptor,
    removeLogoutInterceptor,
    logout,
  };
});
export type LogoutInterceptor = () => MaybePromise<boolean | void>;
